﻿/* 
Скрипт расставляет оригиналы на место превьюшек для некоторых стоков.
(Shutter, Depositphotos, Fotolia, Tass, Lori).
Как работает скрипт:
Скрипт находит в имени файла набор цифр (идентификатор изображения), и пытается перелинковать превью на оригинал.
Если оригинала в папке с заказом не окажется, или если в имени изображения нет ни дефисов ни идентификаторов, превью останется на месте. 
- Скрипт работает только с изображениями jpg. Перелинковываются только jpg на jpg.
- Скрипт не берет в обработку: не jpg, заблокированные объекты, объекты на заблокированных и невидимых слоях,
объекты на полях документа, внедренную графику (embegged) и оригиналы некоторых стоков. 
- Можно запускать скрипт несколько раз для одного и того же документа.
- Если в диалоговом окне поставить галочку «Show report», после завершения работы скрипта 
вы получите список изображений, которые не удалось перелинковать. Этот список можно скопировать в буфер обмена.
- Скрипт ShutterHand перелинкует только изображения только тех фотобанков, которые забиты в скрипте, так уж он устроен. 
- Результат работы скрипта рекомендуется проверить.
Автор: Алексей Стукун, st_verstka@mail.ru.
*/


if (app.documents.length == 0) {
    alert ("No documents are open");
    exit();    
    }

var myDoc = app.activeDocument;
var ErrLog = [];
var SuccLog = 0;

if (File.fs == "Macintosh") {
    var sisSlash = "/";
    } else {
        var sisSlash = "\\"; 
}
// Заполняем список ключевых слов для оригиналов (для того, чтобы не брать в обработку оригиналы):
var OriginalNames = [];
var n = 0;
OriginalNames[n++] = "shutterstock_";
OriginalNames[n++] = "Depositphotos";
OriginalNames[n++] = "Fotolia_";
OriginalNames[n++] = "TASS_";
OriginalNames[n++] = "lori-";
// У Alamy имя превью и имя оригинала совпадают 
// OriginalNames[n++] = "^([A-Z]|\\d)+\\.jpg"; // Alamy // эта строка устарела, но пусть пока повисит так

var OriginalNamesLen = OriginalNames.length; 

myDialog = app.dialogs.add({ name: "Shutter Hand Options:" });
with (myDialog.dialogColumns.add()) {
    var RB = radiobuttonGroups.add();
    with (RB) {
        var RBDoc = radiobuttonControls.add({ staticLabel: "Document" });
        var RBSpread = radiobuttonControls.add({ staticLabel: "Spread", checkedState: true });             
        if (app.selection.length>0){
            var RBSelect = radiobuttonControls.add({ staticLabel: "Selection", checkedState: true });
            }
        staticTexts.add(); 
        var ErrLogBox = dialogRows.add().checkboxControls.add( {staticLabel: "Show report", checkedState: true} );
        }
}
var myResult = myDialog.show();
if (!myResult) { exit(); }

var picFolder = Folder.selectDialog();
if (picFolder == null) { exit(); }

switch (RB.selectedButton) {
    case 0:
    var myImages = myDoc.allGraphics;
    break;
    case 1:
    var myImages = app.activeWindow.activeSpread.allGraphics;
    break;
    case 2:
    var mySelection = app.selection;
    var myImages = new Array ();
    for (s=0; s<(mySelection.length); s++) {
        var myImages = myImages.concat(mySelection[s].allGraphics);
        }
}

var myImagesLen = myImages.length
for (i = 0; i < myImagesLen; i++) {
    var myLink = myImages[i].itemLink;
    var myFrame = myImages[i].parent;
    var myName = myLink.name;
    var myPath = myLink.filePath;
    
    // Исключаем из обработки:
    if (!myFrame.parentPage){ continue; } // На полях
    if (myLink.status == 1282237028) { continue; } // Embegged
    if (myFrame.locked == true) { continue; } // Заблокированные объекты
    if (myFrame.itemLayer.locked == true) { continue; } // Заблокированные слои
    if (myFrame.itemLayer.visible == false) { continue; } // Невидимые слои
    if (myLink.linkType != "JPEG") { continue; } // Не JPG
    if (CheckIfOriginal(myName)) { continue; } // Если имя начинается с OriginalNames
    
    // Задаем данные для стоков, у которых KeyWord - набор цифр.
    var myRegExp = /\d\d\d\d\d+/;
    var myKeyWord = myLink.name.match(myRegExp);
    
    // Пытаемся перелинковать Shutter:
    var myNewPath = picFolder.fsName + sisSlash + "shutterstock_" + myKeyWord + ".jpg";
    if (TryToRelink()) {
        continue;
    }
    // Пытаемся перелинковать Depositphotos:
    var myNewPath = picFolder.fsName + sisSlash + "Depositphotos_" + myKeyWord + "_xl-2015.jpg";
    if (TryToRelink()) {
        continue;
    }

    var myNewPath = picFolder.fsName + sisSlash + "Depositphotos_" + myKeyWord + "_XL.jpg";
    if (TryToRelink()) {
        continue;
    }

    var myNewPath = picFolder.fsName + sisSlash + "Depositphotos_" + myKeyWord + "_original.jpg";
    if (TryToRelink()) {
        continue;
    }
    
    // Пытаемся перелинковать Fotolia:
    var myNewPath = picFolder.fsName + sisSlash + "Fotolia_" + myKeyWord + "_Subscription_Monthly_M.jpg";
    if (TryToRelink()) {
        continue;
    }
    // Пытаемся перелинковать TASS:  
    var myNewPath = picFolder.fsName + sisSlash + "TASS_" + myKeyWord + ".jpg";
    if (TryToRelink()) {
        continue;
    }
    // Пытаемся перелинковать lori:  
    var myNewPath = picFolder.fsName + sisSlash + "lori-" + myKeyWord + "-a5.jpg";
    if (TryToRelink()) {
        continue;
    }
    
    // Пытаемся перелинковать Alamy и вообще все картинки, у которых совпадают имена превью и оригинала:
    var myNewPath = picFolder.fsName + sisSlash + myLink.name;
    if (TryToRelink()) {
        continue;
    }
    
//~     var myRegExp = /([A-Z]|\d)+\./;
//~     var myKeyWord = myLink.name.match(myRegExp);
//~     if (myKeyWord) {
//~         myKeyWord = myKeyWord[0];
//~         myKeyWord = myKeyWord.substring(0, myKeyWord.length-1);
//~         var myNewPath = picFolder.fsName + sisSlash + myKeyWord + ".jpg";
//~         if (TryToRelink()) {
//~             continue;
//~         }
//~     }
   
    // Пишем лог:
    if (ErrLogBox.checkedState == true) {
        ErrLog.push("Page " + myFrame.parentPage.name + ":\r" + myName);
        }

} // END myImagesLen

// Показываем лог:
if (ErrLog.length > 0) {
    ErrLog.sort( function(a, b) { 
        return a.slice(a.indexOf(" "), a.indexOf(":")) - b.slice(b.indexOf(" "), b.indexOf(":")); 
        });    
    app.scriptPreferences.userInteractionLevel = UserInteractionLevels.INTERACT_WITH_ALL;        
    AlertScroll("Relinked: "+SuccLog+" links. Failed: "+ErrLog.length+" links:", ErrLog);
    } else {
        alert ("Relinked items: "+SuccLog);
}

// ==================================================== ФУНКЦИИ: ====================================================

function AlertScroll (title, input) {
    if (input instanceof Array)
    input = input.join ("\r"); 
    var myWindow =    new Window ("dialog", title);
    var myList = myWindow.add ("edittext", undefined, input, {multiline: true, scrolling: true});
    myList.maximumSize.height = 700;
    myList.maximumSize.width = myWindow.maximumSize.width;
    myList.minimumSize.height = 10;
    myList.minimumSize.width = 1200;
    myWindow.add ("button", undefined, "Ok", {name: "Ok"});
    myWindow.show();
}

function CheckIfOriginal(myName) {
    for (k = 0; k < OriginalNamesLen; k++) {
        var myNameRegExp = new RegExp(OriginalNames[k]);
        if (myNameRegExp.test(myName)) {
            return true;
            }
        }    
}

function TryToRelink() {
    try {
        myLink.relink(File(myNewPath));
        ++SuccLog;
        return true;
        } catch (error) {
            }
}